%close all
clear all
%-----------------------------------------------------------------
%Linearize column model using finite differences
%-----------------------------------------------------------------
 %initial conditions for (yd,xb)=(0.97,0.01)
load saved/Xinit001097;
NT=13;
x0 = Xinit(1:NT)';
Mb = Xinit(NT+1);
M1 = Xinit(NT+2:2*NT-1)';
Md = Xinit(2*NT);
T0 = Xinit(2*NT+1:end)';
Lw = 0.52210152164679;
VB = 0.56053583185274;
F = 1.0;
zF = 0.3095;
X0 = [x0,Mb,M1,Md,T0]';
U0 = [Lw VB F zF]';



% %Get linearized system 
% %Note uncomment line 106 in column1.m to solve
% %algebraic equations without use of mass matrix
% [A,B,C,D]=linearize('column1_linearize',[x0,Mb,M1,Md],U0); 
% Gfull = pck(A,B,C,D);
% save saved/Gfull001097 Gfull
load saved/Gfull001097;

%-----------------------------------------------------------------
%Reduce the order of the linearized plant
%-----------------------------------------------------------------
%Full model has 26 states
%Balanced truncation model reduction to 8 states
k = 8; %desired number of states
[sysb,hsig]=sysbal(Gfull); 
Gred = strunc(sysb,k);

%-----------------------------------------------------------------
%Scale models according to maximum input and output variatons
%-----------------------------------------------------------------
yd0 = 0.97; xb0 = 0.01;

%Maximum input usage while avoiding saturation
dLw_max =0.5;
dUb_max =0.5;
dFw_max =1;
dzF_max =0.2;
scale_input = diag([dLw_max dUb_max dFw_max dzF_max]);

%Maximum allowed output variations of 0.005
Yd_max = max(abs([log(1-yd0+0.005)-log(1-yd0),log(1-yd0-0.005)-log(1-yd0)]));
Xb_max = max(abs([log(xb0-0.005)-log(xb0),log(xb0+0.005)-log(xb0)]));
scale_out = diag([1/Yd_max 1/Xb_max]);

G = mmult(scale_out,Gfull,scale_input);
Gr = mmult(scale_out,Gred,scale_input);
[A,B,C,D]=unpck(Gr);

%-----------------------------------------------------------------
%Fitting G to 1.order plus delay and finding Skogestad PI tunings
%-----------------------------------------------------------------
g11 = sel(G,1,1);
g22 = sel(G,2,2);
[a11,b11,c11,d11] = unpck(g11);
[a22,b22,c22,d22] = unpck(g22);
%Exciting with white noise and using ident to find model:
% Process model g11 with transfer function   
%            K                           
% G(s) = ---------- * exp(-Td*s)         
%         1+Tp1*s                        
                          
K = -20.559;                     
Tp1 = 19.643;                      
Td = 1.4887; 
tau1c = -0.5*Td;%+0.5;%Td; %design parameter
kc1 = 1/K*Tp1/(tau1c+Td);
tauI1 = min(Tp1,4*(tau1c+Td));
% Process model g22 with transfer function   
%            K                           
% G(s) = ---------- * exp(-Td*s)         
%         1+Tp1*s                        
K = -144.95;                     
Tp1 = 184.26;                      
Td = 1.5222;      
tau2c = 0*Td;%+0.5; %design parameter
kc2 = 1/K*Tp1/(tau2c+Td);
tauI2 = min(Tp1,4*(tau2c+Td));

Kpi11 = nd2sys([tauI1 1],[tauI1 0],kc1);
Kpi22 = nd2sys([tauI2 1],[tauI2 0],kc2);
Kpi   = daug(Kpi11,Kpi22); 
[api,bpi,cpi,dpi]=unpck(Kpi);
%Kpi = ss(api,bpi,cpi,dpi);

%-----------------------------------------------------------------
%Select loop shaping functions do get desired shape of singular values
%-----------------------------------------------------------------
%wu1 = zp2sys([-0.04 -0.06],[0 -0.01],2);
%wu2 = zp2sys([-0.04 -0.06],[0 -0.01],2);
%wu1 = zp2sys([-0.08 -0.1],[0 -0.04],2);
%wu2 = zp2sys([-0.08 -0.1],[0 -0.04],2);
wu1 = zp2sys([-0.08 -0.1],[0 -0.04],1.5);
wu2 = zp2sys([-0.08 -0.1],[0 -0.04],1.5);

Wu = daug(wu1,wu2);
Wu = Kpi;
wd = 1;
Wd = daug(wd,wd);
W = daug(Wu,Wd);

%-----------------------------------------------------------------
%Create 1-DOF H_infinity loop shaping controller for shaped plant
%-----------------------------------------------------------------
[Kinf,emax]=ncfsyn(sel(mmult(Gr,W),1:2,1:2),1.1);
disp(strcat('obtained emax: ',num2str(emax)))
[ak,bk,ck,dk]=unpck(Kinf);
Kinf0 = ck*inv(ak)*bk+dk;

%-----------------------------------------------------------------
%Prefilter with anti-windup
%-----------------------------------------------------------------
[aw1,bw1,cw1,dw1]=unpck(Wu);
aw = aw1-bw1*inv(dw1)*cw1;
bw = [zeros(size(bw1)) bw1*inv(dw1)];
cw = cw1;
dw = [dw1 zeros(size(dw1))];

%-----------------------------------------------------------------
%Discretize and save controller Kinf, PI compensator and prefilter Kinf0
%-----------------------------------------------------------------
Ts = 2/60; %2sec sampling interval
Kinf_d=c2d(ss(ak,bk,ck,dk),Ts);
Wd=c2d(ss(aw,bw,cw,dw),Ts);
ak_d = Kinf_d.a;
bk_d = Kinf_d.b;
ck_d = Kinf_d.c;
dk_d = Kinf_d.d;
aw_d = Wd.a;
bw_d = Wd.b;
cw_d = Wd.c;
dw_d = Wd.d;

nr = 10;

%save controller state-space matrices for Kinf, W and Kinf0
save(strcat('saved\ak_d',num2str(nr),'.txt'),'ak_d','-ascii', '-double','-tabs')
save(strcat('saved\bk_d',num2str(nr),'.txt'),'bk_d','-ascii', '-double','-tabs')
save(strcat('saved\ck_d',num2str(nr),'.txt'),'ck_d','-ascii', '-double','-tabs')
save(strcat('saved\dk_d',num2str(nr),'.txt'),'dk_d','-ascii', '-double','-tabs')

save(strcat('saved\aw_d',num2str(nr),'.txt'),'aw_d','-ascii', '-double','-tabs')
save(strcat('saved\bw_d',num2str(nr),'.txt'),'bw_d','-ascii', '-double','-tabs')
save(strcat('saved\cw_d',num2str(nr),'.txt'),'cw_d','-ascii', '-double','-tabs')
save(strcat('saved\dw_d',num2str(nr),'.txt'),'dw_d','-ascii', '-double','-tabs')

save(strcat('saved\Kinf0',num2str(nr),'.txt'),'Kinf0','-ascii', '-double','-tabs')

%-----------------------------------------------------------------
%Loop transfer functions
%-----------------------------------------------------------------
Gu = sel(G,1:2,1:2);
GW = mmult(Gu,Wu);
WG = mmult(Wu,Gu);
Lo = mmult(Gu,Wu,Kinf);
Li = mmult(Wu,Kinf,Gu);
GKpi = mmult(Gu,Kpi);
KGpi = mmult(Kpi,Gu);




%-----------------------------------------------------------------
%Additional simulation parameters
%-----------------------------------------------------------------
%input gain perturbations
Kl = 1;%[1 0.5 0.5 1.5 1.5];
Kp = 1;%[1 0.5 1.5 1.5 0.5];
Theta = [0.01 3/6 3/6 3/6 3/6]
%Measurment matrix
C = zeros(2,NT);
C(1,NT)=1; C(2,1)=1;
del_yd = -0.02;%0.01;
del_xb = -0.005;%-0.005;
del_F = 0;%0.3;
del_zf = 0;%1;

%-----------------------------------------------------------------
%Run simulation
%-----------------------------------------------------------------
for i=1:length(Kl)
    i
    kl=Kl(i);kp=Kp(i);theta = Theta(i);
%     U0 = [Lw VB F zF]';
%     U0(1)=U0(1)/kl;
%     U0(2)=U0(2)/kp;
    sim('simcolumn1');
    data = squeeze(simout);
    time{i} = data(1,:);
    yd_ref{i} = 1 - data(2,:);
    xb_ref{i} = data(3,:);
    yd{i} = data(4,:);
    xb{i} = data(5,:);
    L{i} = data(6,:);
    V{i} = data(7,:);
end
%-----------------------------------------------------------------
%Various plots
%-----------------------------------------------------------------
plotplots = 1;
if plotplots==1
%     w = logspace(-3,2,100);
%     G_w = frsp(G,w);
%     Gr_w  = frsp(Gr,w);
%     svdG = vsvd(frsp(Gu,w));
%     svdGW = vsvd(frsp(GW,w));
%     svdWG = vsvd(frsp(WG,w));
%     svdLo = vsvd(frsp(Lo,w));
%     svdLi = vsvd(frsp(Li,w));
%     svdGKpi = vsvd(frsp(GKpi,w));
%     svdKGpi = vsvd(frsp(KGpi,w));
%     svdK = vsvd(frsp(Kinf,w));
%     svdW = vsvd(frsp(Wu,w));
%     figure(2);
%     vplot('liv,lm',svdG,'b--',svdGW,'k-.',svdLo,'r-');
%     xlabel('frequency [rad/min]')
%     ylabel('gain [dB]')
    %title('Output loop transfer function')
    %legend('G','GW_1','GW_1K_{\infty}')

    %     figure(2);vplot('liv,lm',svdG,'b--',svdWG,'k-.',svdLi,'r-');
    %     xlabel('frequency [rad/min]')
    %     ylabel('gain [dB]')
    %     title('Input loop transfer function')
    %     %legend('G','GW_1','GW_1K_{\infty}')
%     figure(4);
%     vplot('liv,lm',svdK,'b-',svdW,'k--');
%     xlabel('frequency [rad/min]')
%     ylabel('gain [dB]')
       for i = 1:length(Kl)
%         figure(5);
%         hold on;
%         plot(time{i},yd_ref{i}-0.97,'b--',time{i},yd{i}-0.97,'k-',time{i},xb_ref{i}-0.01,'b--',time{i},xb{i}-0.01,'k-')
%         xlabel('time [min]')
%         ylabel('change in product compositions')

         figure(3);
        subplot(2,1,1)
         if i==1
             plot(time{i},yd_ref{i},'b--')
             hold on
             plot(time{i},yd{i},'k-','LineWidth',2)
         else
         hold on;
        plot(time{i},yd{i},'k-')
         end
        xlabel('time [min]')
        ylabel('y_d')
        axis([0 70 0.9 0.98])
        subplot(2,1,2)
        if i==1
             plot(time{i},xb_ref{i},'b--')
             hold on
             plot(time{i},xb{i},'k-','LineWidth',2)
         else
         hold on;
          plot(time{i},xb{i},'k-')
         end
           
        xlabel('time [min]')
        ylabel('x_b')
        axis([0 70 0.0 0.02])
        figure(2);
        subplot(2,1,1)
        hold on;
        plot(time{i},L{i});
        ylabel('reflux rate [ml/s]')
        subplot(2,1,2)
        hold on;
        plot(time{i},3*V{i})
        xlabel('time [min]')
        ylabel('heat power [kW]')
     end


end




